﻿using System;
using System.Windows;
using System.Windows.Interactivity;
using Infragistics.DragDrop;

namespace DragDropFrameworkPOC
{
  public class DragDropBehavior : Behavior<FrameworkElement>
  {
    #region Behaviour Overrides

    /// <summary>
    /// On behavior getting attached,
    /// </summary>
    protected override void OnAttached()
    {
      if (null != AssociatedObject.DataContext)
      {
        var dragSource = AssociatedObject.DataContext as IDragSource;
        if (null != dragSource)
        {
          var infragisticsDragSource = new DragSource() { IsDraggable = dragSource.IsDraggable, DragChannels = dragSource.DragChannels };
          infragisticsDragSource.Drop += DragSource_Drop;
          DragDropManager.SetDragSource(AssociatedObject, infragisticsDragSource);
        }

        var dropTarget = AssociatedObject.DataContext as IDropTarget;
        if (null != dropTarget)
        {
          var infragisticsDropTarget = new DropTarget() { DropChannels = dropTarget.DropChannels, IsDropTarget = dropTarget.IsDropTarget };
        }
      }
      else
      {
        AssociatedObject.DataContextChanged += AssociatedObject_DataContextChanged;
      }
    }

    void AssociatedObject_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
    {
      var dragSource = AssociatedObject.DataContext as IDragSource;
      if (null != dragSource)
      {
        var infragisticsDragSource = new DragSource() { IsDraggable = dragSource.IsDraggable, DragChannels = dragSource.DragChannels };
        infragisticsDragSource.Drop += DragSource_Drop;
        DragDropManager.SetDragSource(AssociatedObject, infragisticsDragSource);
      }

      var dropTarget = AssociatedObject.DataContext as IDropTarget;
      if (null != dropTarget)
      {
        var infragisticsDropTarget = new DropTarget() { DropChannels = dropTarget.DropChannels, IsDropTarget = dropTarget.IsDropTarget };
      }
    }

    private void DragSource_Drop(object sender, DropEventArgs e)
    {
      foreach (FrameworkElement fe in e.DropTargetElements)
      {
        var dropTarget = fe.DataContext as IDropTarget;
        if (sender is IDragSource)
          dropTarget.OnDropped((sender as IDragSource).DropArgument);
      }
    }

    /// <summary>
    /// While the behavior is getting detached.
    /// </summary>
    protected override void OnDetaching()
    {
      AssociatedObject.DataContextChanged -= AssociatedObject_DataContextChanged;
      base.OnDetaching();

    }
    #endregion
  }


  public static class DragDropAttached
  {
    #region IsBroughtIntoViewWhenSelected


    public static readonly DependencyProperty IsDragDropProperty =
        DependencyProperty.RegisterAttached(
        "IsDragDrop",
        typeof(bool),
        typeof(DragDropAttached),
        new UIPropertyMetadata(false, OnIsDragDropChanged));

    public static bool GetIsDragDropProperty(FrameworkElement attachedElement)
    {
      return (bool)attachedElement.GetValue(IsDragDropProperty);
    }

    public static void SetIsDragDropProperty(
      FrameworkElement attachedElement, bool value)
    {
      attachedElement.SetValue(IsDragDropProperty, value);
    }

    static void OnIsDragDropChanged(
      DependencyObject depObj, DependencyPropertyChangedEventArgs e)
    {
      var item = depObj as FrameworkElement;
      if (item == null)
        return;

      if (e.NewValue is bool == false)
        return;
      else
      {
        if (null != item.DataContext)
        {
          var dragSource = item.DataContext as IDragSource;
          if (null != dragSource)
          {
            var infragisticsDragSource = new DragSource() { IsDraggable = dragSource.IsDraggable, DragChannels = dragSource.DragChannels };
            infragisticsDragSource.Drop += DragSource_Drop;
            DragDropManager.SetDragSource(item, infragisticsDragSource);
          }

          var dropTarget = item.DataContext as IDropTarget;
          if (null != dropTarget)
          {
            var infragisticsDropTarget = new DropTarget() { DropChannels = dropTarget.DropChannels, IsDropTarget = dropTarget.IsDropTarget };
            DragDropManager.SetDropTarget(item, infragisticsDropTarget);
          }
        }
        else
        {
          item.DataContextChanged += AssociatedObject_DataContextChanged;
        }
      }
    }

    static void AssociatedObject_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
    {
      var associatedObject = sender as FrameworkElement;
      if(null!=associatedObject)
      {
        var dragSource = associatedObject.DataContext as IDragSource;
        if (null != dragSource)
        {
          var infragisticsDragSource = new DragSource()
            {
              IsDraggable = dragSource.IsDraggable,
              DragChannels = dragSource.DragChannels
            };
          infragisticsDragSource.Drop += DragSource_Drop;
          DragDropManager.SetDragSource(associatedObject, infragisticsDragSource);
        }

        var dropTarget = associatedObject.DataContext as IDropTarget;
        if (null != dropTarget)
        {
          var infragisticsDropTarget = new DropTarget()
            {
              DropChannels = dropTarget.DropChannels,
              IsDropTarget = dropTarget.IsDropTarget
            };
          DragDropManager.SetDropTarget(associatedObject, infragisticsDropTarget);
        }
      }
    }

    private static void DragSource_Drop(object sender, DropEventArgs e)
    {
        foreach (FrameworkElement fe in e.DropTargetElements)
        {
            var dropTarget = fe.DataContext as IDropTarget;
            FrameworkElement dropElement = (sender as DragSource).AssociatedObject as FrameworkElement;
            if (dropElement != null && dropElement.DataContext is IDragSource)
                dropTarget.OnDropped((dropElement.DataContext as IDragSource).DropArgument);
        }
        //foreach (FrameworkElement fe in e.DropTargetElements)
        //{
        //    var dropTarget = fe.DataContext as IDropTarget;
        //    if (sender is IDragSource)
        //        dropTarget.OnDropped((sender as IDragSource).DropArgument);
        //}
    }

    #endregion // IsBroughtIntoViewWhenSelected
  }
}
